@@ -1,4 +1,4 @@
-from __future__ import absolute_import
+from __future__ import absolute_import, division, print_function, unicode_literals, with_statement
 
 import base64
 from copy import deepcopy
@@ -269,7 +269,7 @@ class LDAPObject(RecordableMethods):
                    if filter_expr.matches(dn, self.directory[dn]))
 
         # Apply attribute filtering, if any
-        if attrlist is not None:
+        if (attrlist is not None) and ('*' not in attrlist):
             results = ((dn, dict((attr, values) for attr, values in attrs.items() if attr in attrlist))
                        for dn, attrs in results)
 
@@ -292,7 +292,7 @@ class LDAPObject(RecordableMethods):
 
             if value is None:
                 value = []
-            elif type(value) is str:
+            elif not isinstance(value, list):
                 value = [value]
 
             if op == ldap.MOD_ADD:
@@ -360,17 +360,17 @@ class LDAPObject(RecordableMethods):
         oldattr, oldvalue = dn.split(',')[0].split('=')
         newattr, newvalue = newrdn.split('=')
 
+        if oldattr == newattr or len(entry[oldattr]) > 1:
+            entry[oldattr].remove(oldvalue)
+        else:
+            del entry[oldattr]
+
         try:
             if newvalue not in entry[newattr]:
                 entry[newattr].append(newvalue)
         except KeyError:
             entry[newattr] = [newvalue]
 
-        if oldattr == newattr or len(entry[oldattr]) > 1:
-            entry[oldattr].remove(oldvalue)
-        else:
-            del entry[oldattr]
-
         self.directory[newfulldn] = entry
         del self.directory[dn]
 
